// ==============================================================
// $Id: BurnTimeMFD.cpp,v 1.4 2004/05/25 19:08:46 dh219 Exp $
// BTC by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define RED RGB(255, 0, 0)
#define YELLOW RGB(255, 255, 0)
#define WHITE RGB(255, 255, 255)
#define BLUE RGB(0, 0, 255)
#define GREY RGB(160, 160, 160)
#define BRIGHTERGREY RGB(200, 200, 200)

#include "../../include/orbitersdk.h"
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "MavCode.h"

class BurnTimeMFD: public MFD {
public:
	BurnTimeMFD (DWORD w, DWORD h, VESSEL *vessel);
	~BurnTimeMFD ();
	bool ConsumeKeyBuffered (DWORD key);
	void Update (HDC hDC);
	static int MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam);
	int ButtonMenu (const MFDBUTTONMENU **menu) const;
	char *ButtonLabel (int bt);
	bool ConsumeButton(int bt, int event);

protected:

	VESSEL *messel;
	int height;


private:
	void writemenu( HDC hDC );
	int line( int );

};

double dv;

static struct {
	int mode;
} g_BurnTimeMFD;



BurnTimeMFD *burntime;


DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	static char *name = "Burn Time MFD";
	MFDMODESPEC spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_T;
	spec.msgproc = BurnTimeMFD::MsgProc;

	g_BurnTimeMFD.mode = oapiRegisterMFDMode (spec);

	burntime = NULL;
	dv = 0.0;
}


DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
	oapiUnregisterMFDMode (g_BurnTimeMFD.mode);
}

DLLCLBK void opcTimestep (double simt, double simdt, double mjd)
{


}

BurnTimeMFD::BurnTimeMFD (DWORD w, DWORD h, VESSEL *vessel)
: MFD (w, h, vessel)
{
	height = (int)h;

}
BurnTimeMFD::~BurnTimeMFD ()
{


}
void BurnTimeMFD::Update (HDC hDC)
{
	char buffer[80];
	OBJHANDLE obj;
	VESSEL *vessel;
	THRUSTER_HANDLE th[10];
	unsigned int thrustercount = 0;
	double T;
	double m;
	double isp;
	double F;


	Title (hDC, "Burn Time Calculator");

	SetTextColor( hDC, YELLOW );
	sprintf( buffer, "Target dV: %5.1lf", dv );
	TextOut( hDC, 20, line(4), buffer, strlen(buffer));
	
	SetTextColor( hDC, WHITE );

	obj = oapiGetFocusObject();
	vessel = oapiGetVesselInterface(obj);

	for( thrustercount = 0 ; thrustercount < 10 && ( th[thrustercount] = mavGetGroupThusterByIndex( vessel, THGROUP_MAIN, (int)thrustercount ) ) != NULL ; thrustercount++ )
		;

//	isp = vessel->GetThrusterIsp( th[0] );
	F = 0;
	isp = 0;
	unsigned int i;
	for( i = 0 ; i < thrustercount ; i++ ) {
		F += vessel->GetThrusterMax0( th[i] );
		isp += vessel->GetThrusterIsp( th[i] ) * vessel->GetThrusterMax0( th[i] );
	}
	m = vessel->GetMass();
	isp /= F;

	T = ( dv * m / (2.0 * F ) ) * ( 1 + exp( -1.0 * dv / isp ) );

	double dv2 = dv/2.0;
	double T2;
	T2 = ( dv2 * m / (2.0 * F ) ) * ( 1 + exp( -1.0 * dv2 / isp ) );

	double maxdv = isp * log( m / vessel->GetEmptyMass() );

	sprintf( buffer, "Estimated burn time: %5.1f seconds", T );
	TextOut( hDC, 20, line(8), buffer, strlen(buffer));
	sprintf( buffer, "Engage at PeT =      %5.1f seconds", T2 );
	TextOut( hDC, 20, line(10), buffer, strlen(buffer));

	SetTextColor( hDC, BRIGHTERGREY );
	sprintf( buffer, "Estimate Total dV =  %5.0f m/s", maxdv );
	TextOut( hDC, 20, line(12), buffer, strlen(buffer));


	SetTextColor( hDC, GREY );
	sprintf( buffer, "Keys: -/+  [/] </>" );
	TextOut( hDC, 20, line(14), buffer, strlen(buffer));
	sprintf( buffer, "      BKSPC = reset" );
	TextOut( hDC, 20, line(15), buffer, strlen(buffer));

}
int BurnTimeMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
		case OAPI_MSG_MFD_OPENED:
			burntime = (new BurnTimeMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
			return (int)burntime;
	}

	return 0;
}

bool BurnTimeMFD::ConsumeKeyBuffered (DWORD key)
{
	
	if( key == OAPI_KEY_MINUS ) {
		dv -= 100.0;
		return true;
	}
	if( key == OAPI_KEY_EQUALS ) {
		dv += 100.0;
		return true;
	}
	if( key == OAPI_KEY_LBRACKET ) {
		dv -= 10.0;
		return true;
	}
	if( key == OAPI_KEY_RBRACKET ) {
		dv += 10.0;
		return true;
	}
	if( key == OAPI_KEY_COMMA ) {
		dv -= 1.0;
		return true;
	}
	if( key == OAPI_KEY_PERIOD ) {
		dv += 1.0;
		return true;
	}
	if( key == OAPI_KEY_BACK ) {
		dv = 0.0;
		return true;
	}


	return false;
}

int BurnTimeMFD::line( int i ) {

	return (int)((float)i*((float)height/20.0));

}

int BurnTimeMFD::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	static const MFDBUTTONMENU mnu[7] = {
		{"Up   100", 0, '-'},
		{"Down 100", 0, '='},
		{"Up    10", 0, '['},
		{"Down  10", 0, ']'},
		{"Up     1", 0, ','},
		{"Downl  1", 0, '.'},
		{"Reset", 0, '<' }
	};
	if (menu) *menu = mnu;
	return 7;
}

char *BurnTimeMFD::ButtonLabel (int bt)
{
	char *label[11] = {"+++","++","+", "","","", "---", "--","-", "RST",""};
	return (bt < 11 ? label[bt] : 0);
}

bool BurnTimeMFD::ConsumeButton(int bt, int event) {

	if (event & PANEL_MOUSE_LBDOWN) {

		switch( bt ) {
		case(0):
			return ConsumeKeyBuffered (OAPI_KEY_EQUALS);
		case(1):
			return ConsumeKeyBuffered (OAPI_KEY_RBRACKET);
		case(2):
			return ConsumeKeyBuffered (OAPI_KEY_PERIOD);
		case(6):
			return ConsumeKeyBuffered (OAPI_KEY_MINUS);
		case(7):
			return ConsumeKeyBuffered (OAPI_KEY_LBRACKET);
		case(8):
			return ConsumeKeyBuffered (OAPI_KEY_COMMA);
		case(9):
			return ConsumeKeyBuffered (OAPI_KEY_BACK);
		default:
			return false;
		}
	}

	return false;
}
